/*************************************************************************************
* Final Project - Multi-Layered Travel Activity Networks
* 
*
* 
* Author:    Su Burtner                       
* 
* Modified:  March 8th, 2019
* 
* Data:      This data comes from the California Household Travel Activity Survey
*            (http://www.dot.ca.gov/hq/tpp/offices/omsp/statewide_travel_analysis/chts.html)
*            and contains over 40,000 records for where people went at what time and for how
*            long across either one day or 3 day periods, depending on the collection method.
*
* Query      Top 50 individuals with the most travel activity by race living in Los Angeles or
*            Orange County.
* 
* Purpose:   Create a multi-layered network that shows different patterns of travel activity
*            among different racial groups in Los Angeles.
* 
* Usage:     Default is to see travel separated by race, press any key that is not "d" or "m"
*            to return to this.
*            Press 'd' to see travel separated by days of the week.
*            Press 'm' to see travel separated by mode of transit.
*            Press 'h' at any time to toggle drawing the convex hull.
*            Check "Presentation Mode" to rotate scene.
*
* Test:      Processing 3
* 
* Note:      This work builds off of previous assignments, which also build off of
*            other code in other projects. I especially want to acknowledge and thank
*            Rodger Luo (TA for MAT 259, Winter 2019) and George Legrady (Professor of
*            MAT 259, Winter 2019) for their help.
*
**************************************************************************************/

//-----Import libraries----------------------------------------------------------------
import org.gicentre.geomap.*;         // mapping library
import org.gicentre.utils.spatial.*;  // spatial library
import ComputationalGeometry.*;       // to create the convex hulls
//import quickhull3d.*;
//import template.library.*;
import peasy.*;                       // 3D Navigation library
import controlP5.*;                   // GUI library

//-----Constants Section---------------------------------------------------------------
                             
final int LAYER_DIST = 300;          //Distance between each layer in virtual space 
final int LAYER_DIST_DOW = 200;
final int ZFACTOR = 5;

//-----Variables Section---------------------------------------------------------------
boolean drawRace = true;
boolean drawDOW = false;
boolean drawMode = false;
boolean addHull = false;

boolean presentationMode = false;    // true -> automation 
                                     // false -> no automation

PeasyCam cam;                        //Camera(3D Nav) variable

ControlP5 cp5;                       //User Interface 

// Variables for data
Table data;
int rows, cols;
String[][] data_matrix;

ArrayList<PVector> coordsW, coordsH, coordsB, coordsA;
ArrayList<PVector> coordsMon, coordsTues, coordsWed, coordsThurs, coordsFri, coordsSat, coordsSun;
ArrayList<PVector> coordsAuto, coordsWalk, coordsLocal, coordsBike;

PVector ULCorner, BRCorner;

GeoMap geoMap;

IsoWrap wrapW, wrapH, wrapB, wrapA;
IsoWrap wrapMon, wrapTues, wrapWed, wrapThurs, wrapFri, wrapSat, wrapSun;
IsoWrap wrapAuto, wrapWalk, wrapLocal, wrapBike;

//---------------------------------------------------------------------------------------------
void setup() {
  //size(1920, 1080, P3D);
  
  fullScreen(P3D);
  smooth();
  
  // Map settings
  geoMap = new GeoMap(this);  // Create the geoMap object.
  geoMap.readFile("la-county-neighborhoods-v6/l.a. county neighborhood (v6)");   // Reads shapefile.
  
  //Initial camera and user interface 
  cam = new PeasyCam(this, 2000);
  cp5 = new ControlP5(this);
  
  cp5.addCheckBox("checkBox")
     .setPosition(20, 10)
     .setSize(15, 15) 
     .addItem("Presentation Mode", 1)
     .activateAll(); //This function makes the check box selected by default
     
  cp5.setAutoDraw(false);
  
  // Put data into tables
  data = loadTable("top50_la_travel.csv", "header");
  
  rows = data.getRowCount();
  cols = data.getColumnCount();
  
  data_matrix = new String[rows][cols];
  
  // Read in the data
  AlbersUS proj = new AlbersUS();
   
  // Get data from table
  for (int i=0; i < data_matrix.length; i++) {
    for (int j=0; j < data_matrix[0].length; j++) {
      data_matrix[i][j] = data.getString(i, j);
    }
  }
  
  // NOTE: the bottom code could all be in one series of "if" statements, but I have separate them out for clarity
  // Create coordinate arrays for each race
  coordsW = new ArrayList<PVector>();
  coordsH = new ArrayList<PVector>();
  coordsB = new ArrayList<PVector>();
  coordsA = new ArrayList<PVector>();
  
  for(int i=0; i < data_matrix.length; i++) {
    float lon = float(data_matrix[i][13]);
    float lat = float(data_matrix[i][14]);
    int timeArr = int(data_matrix[i][15]);
    if(data_matrix[i][5].equals("White")) {
      PVector tmpW = proj.transformCoords(new PVector(lon, lat));
      coordsW.add(new PVector(tmpW.x, tmpW.y, timeArr));
      //coordsW.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    } else if(data_matrix[i][5].equals("Hispanic")) {
      PVector tmpH = proj.transformCoords(new PVector(lon, lat));
      coordsH.add(new PVector(tmpH.x, tmpH.y, timeArr));
      //coordsH.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    } else if(data_matrix[i][5].equals("Black or African American")) {
      PVector tmpA = proj.transformCoords(new PVector(lon, lat));
      coordsA.add(new PVector(tmpA.x, tmpA.y, timeArr));
      //coordsA.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    } else if(data_matrix[i][5].equals("Asian (Asian Indian, Japanese, Chinese, Korean, Filipino, Vietnamese)")) {
      PVector tmpB = proj.transformCoords(new PVector(lon, lat));
      coordsB.add(new PVector(tmpB.x, tmpB.y, timeArr));
      //coordsB.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    }
  }
  
  // Create coordinate arrays for days of the week
  coordsMon = new ArrayList<PVector>();
  coordsTues = new ArrayList<PVector>();
  coordsWed = new ArrayList<PVector>();
  coordsThurs = new ArrayList<PVector>();
  coordsFri = new ArrayList<PVector>();
  coordsSat = new ArrayList<PVector>();
  coordsSun = new ArrayList<PVector>();
  
  for(int i=0; i < data_matrix.length; i++) {
    float lon = float(data_matrix[i][13]);
    float lat = float(data_matrix[i][14]);
    int timeArr = int(data_matrix[i][15]);
    if(data_matrix[i][35].equals("Monday")){
      PVector tmpMon = proj.transformCoords(new PVector(lon, lat));
      coordsMon.add(new PVector(tmpMon.x, tmpMon.y, timeArr));
      //coordsMon.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    } else if(data_matrix[i][35].equals("Tuesday")) {
      PVector tmpTues = proj.transformCoords(new PVector(lon, lat));
      coordsTues.add(new PVector(tmpTues.x, tmpTues.y, timeArr));
      //coordsTues.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    } else if(data_matrix[i][35].equals("Wednesday")) {
      PVector tmpWed = proj.transformCoords(new PVector(lon, lat));
      coordsWed.add(new PVector(tmpWed.x, tmpWed.y, timeArr));
      //coordsWed.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    } else if(data_matrix[i][35].equals("hursday")) { // weird spelling error in data
      PVector tmpThurs = proj.transformCoords(new PVector(lon, lat));
      coordsThurs.add(new PVector(tmpThurs.x, tmpThurs.y, timeArr));
      //coordsThurs.add(proj.transformCoords(new PVector(lon, lat, timeArr)));
    } else if(data_matrix[i][35].equals("Friday")) {
      PVector tmpFri = proj.transformCoords(new PVector(lon, lat));
      coordsFri.add(new PVector(tmpFri.x, tmpFri.y, timeArr));
      //coordsFri.add(proj.transformCoords(new PVector(lon, lat)));
    } else if(data_matrix[i][35].equals("Saturday")) {
      PVector tmpSat = proj.transformCoords(new PVector(lon, lat));
      coordsSat.add(new PVector(tmpSat.x, tmpSat.y, timeArr));
      //coordsSat.add(proj.transformCoords(new PVector(lon, lat)));
    } else if(data_matrix[i][35].equals("Sunday")) {
      PVector tmpSun = proj.transformCoords(new PVector(lon, lat));
      coordsSun.add(new PVector(tmpSun.x, tmpSun.y, timeArr));
      //coordsSun.add(proj.transformCoords(new PVector(lon, lat)));
    }
  }
  
  // Create coordinate arrays for mode of transportation
  coordsAuto = new ArrayList<PVector>();
  coordsWalk = new ArrayList<PVector>();
  coordsLocal = new ArrayList<PVector>();
  coordsBike = new ArrayList<PVector>();
  
  for(int i=0; i < data_matrix.length; i++) {
    float lon = float(data_matrix[i][13]);
    float lat = float(data_matrix[i][14]);
    int timeArr = int(data_matrix[i][15]);
    if(data_matrix[i][7].equals("Auto / Van / Truck Driver") ||
        data_matrix[i][7].equals("Auto / Van / Truck Passenger")) {
      PVector tmpAuto = proj.transformCoords(new PVector(lon, lat));
      coordsAuto.add(new PVector(tmpAuto.x, tmpAuto.y, timeArr));
      //coordsAuto.add(proj.transformCoords(new PVector(lon, lat)));
    } else if(data_matrix[i][7].equals("Walk")) {
      PVector tmpWalk = proj.transformCoords(new PVector(lon, lat));
      coordsWalk.add(new PVector(tmpWalk.x, tmpWalk.y, timeArr));
      //coordsWalk.add(proj.transformCoords(new PVector(lon, lat)));
    } else if(data_matrix[i][7].equals("Local Bus, Rapid Bus")) {
      PVector tmpLocal = proj.transformCoords(new PVector(lon, lat));
      coordsLocal.add(new PVector(tmpLocal.x, tmpLocal.y, timeArr));
      //coordsLocal.add(proj.transformCoords(new PVector(lon, lat)));
    } else if(data_matrix[i][7].equals("Bike")) {
      PVector tmpBike = proj.transformCoords(new PVector(lon, lat));
      coordsBike.add(new PVector(tmpBike.x, tmpBike.y, timeArr));
      //coordsBike.add(proj.transformCoords(new PVector(lon, lat)));
    }
  }
  
  // Store the Albers US coordinates of the corner of the map.
  //ULCorner = proj.transformCoords(new PVector(-129.07930, 52.22642 )); //calculated in R
  //BRCorner = proj.transformCoords(new PVector(17.09585, 28.89033));
  ULCorner = proj.transformCoords(new PVector(geoMap.getMinGeoX(), geoMap.getMinGeoY()));
  BRCorner = proj.transformCoords(new PVector(geoMap.getMaxGeoX(), geoMap.getMaxGeoY()));
  //println("UL: " + ULCorner.x + ", " + ULCorner.y);
  //println("BR: " + BRCorner.x + ", " + BRCorner.y);
  
  // Create "wraps" aka convex hulls
  wrapW = new IsoWrap(this);
  wrapH = new IsoWrap(this);
  wrapA = new IsoWrap(this);
  wrapB = new IsoWrap(this);
  
  createHull(coordsW, wrapW, ZFACTOR);
  createHull(coordsH, wrapH, ZFACTOR);
  createHull(coordsA, wrapA, ZFACTOR);
  createHull(coordsB, wrapB, ZFACTOR);
  
  wrapMon = new IsoWrap(this);
  wrapTues = new IsoWrap(this);
  wrapWed = new IsoWrap(this);
  wrapThurs = new IsoWrap(this);
  wrapFri = new IsoWrap(this);
  wrapSat = new IsoWrap(this);
  wrapSun = new IsoWrap(this);
  
  createHull(coordsMon, wrapMon, ZFACTOR);
  createHull(coordsTues, wrapTues, ZFACTOR);
  createHull(coordsWed, wrapWed, ZFACTOR);
  createHull(coordsThurs, wrapThurs, ZFACTOR);
  createHull(coordsFri, wrapFri, ZFACTOR);
  createHull(coordsSat, wrapSat, ZFACTOR);
  createHull(coordsSun, wrapSun, ZFACTOR);
  
  wrapAuto = new IsoWrap(this);
  wrapWalk = new IsoWrap(this);
  wrapLocal = new IsoWrap(this);
  wrapBike = new IsoWrap(this);
  
  createHull(coordsAuto, wrapAuto, ZFACTOR);
  createHull(coordsWalk, wrapWalk, ZFACTOR);
  createHull(coordsLocal, wrapLocal, ZFACTOR);
  createHull(coordsBike, wrapBike, ZFACTOR);
}

//----------------------------------------------------------------------------------------------- //

// Draw
void draw() {
  background(0);
  lights();  
  rotateX(PI/4);
  rotateZ(PI/4);
  
  //Set automation in presentation mode 
  if(presentationMode) {
    rotateZ(frameCount*0.0015);
  }
  
  if(drawRace) {
        
    // First layer (White)
    drawNetwork(LAYER_DIST, 0, 234, 115, 23, 110, coordsW, "White", ZFACTOR);
    
    // Second layer (Hispanic)
    drawNetwork(LAYER_DIST, 1, 258, 198, 1, 110, coordsH, "Hispanic", ZFACTOR);
    
    // Third layer (Asian)
    drawNetwork(LAYER_DIST, 2, 115, 218, 184, 110, coordsA, "Asian", ZFACTOR);
    
    // Fourth layer (Black)
    drawNetwork(LAYER_DIST, 3, 35, 100, 170, 110, coordsB, "Black", ZFACTOR);
    
      if(addHull) {
      // First layer (White)
      drawHull(LAYER_DIST, 0, 234, 115, 23, 110, coordsW, wrapW, "White", ZFACTOR);
   
      // Second layer (Hispanic)
      drawHull(LAYER_DIST, 1, 258, 198, 1, 110, coordsH, wrapH, "Hispanic", ZFACTOR);
    
      // Third layer (Asian)
      drawHull(LAYER_DIST, 2, 115, 218, 184, 110, coordsA, wrapA, "Asian", ZFACTOR);
    
      // Fourth layer (Black)
      drawHull(LAYER_DIST, 3, 35, 100, 170, 110, coordsB, wrapB, "Black", ZFACTOR);
    }
  }
  
  if(drawDOW) {
    // First layer (Monday)
    drawNetwork(LAYER_DIST_DOW, 0, 136, 162, 170, 110, coordsMon, "Monday", ZFACTOR);
    
    // Second layer (Tuesday)
    drawNetwork(LAYER_DIST_DOW, 1, 145, 77, 118, 110, coordsTues, "Tuesday", ZFACTOR);
    
    // Third layer (Wednesday)
    drawNetwork(LAYER_DIST_DOW, 2, 29, 78, 137, 110, coordsWed, "Wednesday", ZFACTOR);
    
    // Fourth layer (Thursday)
    drawNetwork(LAYER_DIST_DOW, 3, 0, 178, 202, 110, coordsThurs, "Thursday", ZFACTOR);
    
    // Fifth layer (Friday)
    drawNetwork(LAYER_DIST_DOW, 4, 247, 111, 93, 110, coordsFri, "Friday", ZFACTOR);
    
    // Sixth layer (Saturday)
    drawNetwork(LAYER_DIST_DOW, 5, 125, 207, 182, 110, coordsSat, "Saturday", ZFACTOR);
    
    // Seventh layer (Sunday)
    drawNetwork(LAYER_DIST_DOW, 6, 251, 209, 162, 110, coordsSun, "Sunday", ZFACTOR);
    
    if(addHull) {
      // First layer (Monday)
      drawHull(LAYER_DIST_DOW, 0, 136, 162, 170, 110, coordsMon, wrapMon, "Monday", ZFACTOR);
      
      // Second layer (Tuesday)
      drawHull(LAYER_DIST_DOW, 1, 145, 77, 118, 110, coordsTues, wrapTues, "Tuesday", ZFACTOR);
      
      // Third layer (Wednesday)
      drawHull(LAYER_DIST_DOW, 2, 29, 78, 137, 110, coordsWed, wrapWed, "Wednesday", ZFACTOR);
      
      // Fourth layer (Thursday)
      drawHull(LAYER_DIST_DOW, 3, 0, 178, 202, 110, coordsThurs, wrapThurs, "Thursday", ZFACTOR);
      
      // Fifth layer (Friday)
      drawHull(LAYER_DIST_DOW, 4, 247, 111, 93, 110, coordsFri, wrapFri, "Friday", ZFACTOR);
      
      // Sixth layer (Saturday)
      drawHull(LAYER_DIST_DOW, 5, 125, 207, 182, 110, coordsSat, wrapSat, "Saturday", ZFACTOR);
      
      // Seventh layer (Sunday)
      drawHull(LAYER_DIST_DOW, 6, 251, 209, 162, 110, coordsSun, wrapSun, "Sunday", ZFACTOR);
    }
  }
  
  if(drawMode) {
    // First layer, auto 
    drawNetwork(LAYER_DIST, 0, 86, 163, 166, 110, coordsAuto,  "Auto (Driver or Passenger)", ZFACTOR);
    
    // Second layer, local
    drawNetwork(LAYER_DIST, 1, 227, 181, 5, 110, coordsLocal, "Local or Rapid Bus", ZFACTOR);
    
    // Second layer, walk 
    drawNetwork(LAYER_DIST, 2, 219, 80, 74, 110, coordsWalk, "Walk", ZFACTOR);
    
    // Second layer, bike 
    drawNetwork(LAYER_DIST, 3, 216, 100, 220, 110, coordsBike, "Bike", ZFACTOR);
    
    if(addHull) {
      // First layer, auto 
      drawHull(LAYER_DIST, 0, 86, 163, 166, 110, coordsAuto,  wrapAuto, "Auto (Driver or Passenger)", ZFACTOR);
      
      // Second layer, local
      drawHull(LAYER_DIST, 1, 227, 181, 5, 110, coordsLocal, wrapLocal, "Local or Rapid Bus", ZFACTOR);
      
      // Second layer, walk 
      drawHull(LAYER_DIST, 2, 219, 80, 74, 110, coordsWalk, wrapWalk, "Walk", ZFACTOR);
      
      // Second layer, bike 
      drawHull(LAYER_DIST, 3, 216, 100, 220, 110, coordsBike, wrapBike, "Bike", ZFACTOR);
    }
  }
  
  
  // Draw text bar at top
  cam.beginHUD();
  
  fill(150);
  noStroke();
  rect(0, 0, width, 40);
  fill(0);
  textAlign(RIGHT);
  textSize(16);
  text("Examining segregrated travel behavior in Los Angeles & Orange County", width - 400, 20);
  
  cp5.draw();
  cam.endHUD();
 
}

// Method for presentation mode
void checkBox(float[] a) {
  if(a[0] == 1) presentationMode = false;
  else presentationMode = true;
}

// Convert from Albers US coordinates to screen coordinates.
PVector geoToScreen(PVector geo) {
  return new PVector(map(geo.x, ULCorner.x, BRCorner.x, 0, width),
                     map(geo.y, ULCorner.y, BRCorner.y, 0, height));
}

// Draw network
void drawNetwork(int layerDistVal, int layerI, int strokeR, int strokeG, int strokeB, int strokeAlpha,
                 ArrayList<PVector> coordsAL, String layerName, int ZFACTOR) {
  pushMatrix();
  translate(0, 0, layerDistVal*layerI);
  
  noFill();
  stroke(strokeR, strokeG, strokeB, strokeAlpha);
  strokeWeight(2);
  
  beginShape();
  
  for (PVector coord : coordsAL) {
    PVector scrCoord = geoToScreen(coord);
    vertex(scrCoord.x, scrCoord.y, coord.z*ZFACTOR);
  }
  
  endShape();
  
  // Add text layer name
  cam.beginHUD();
  
  fill(255);
  textAlign(LEFT);
  textSize(12);
  text(layerName, 40, 200-layerI*20);
  
  cp5.draw();
  cam.endHUD();
  
  
  popMatrix();
}

// Create hull
void createHull(ArrayList<PVector> coordsAL, IsoWrap wrapAL, int ZFACTOR) { 
  for(PVector coord : coordsAL) {
    PVector scrCoord = geoToScreen(coord);
    if(coord.x < width && scrCoord.x > 0 && // keeps hull within bounds of LA
      scrCoord.y < height && scrCoord.y > 0) {
        wrapAL.addPt(new PVector(scrCoord.x, scrCoord.y, coord.z*ZFACTOR));
      }
  }
}

void drawHull(int layerDistVal, int layerI, int strokeR, int strokeG, int strokeB, int strokeAlpha,
              ArrayList<PVector> coordsAL, IsoWrap wrapAL, String layerName, int ZFACTOR) {
  // Test convex hull
  pushMatrix();
  translate(0, 0, layerDistVal*layerI);
  
  // Draw the network
  noFill();
  stroke(strokeR, strokeG, strokeB, strokeAlpha);
  strokeWeight(2);
  
  beginShape();
  
  for (PVector coord : coordsAL) {
    PVector scrCoord = geoToScreen(coord);
    vertex(scrCoord.x, scrCoord.y, coord.z*ZFACTOR);
  }
  
  endShape();
  
  // Draw the hull
  fill(strokeR, strokeG, strokeB, strokeAlpha);
  stroke(100);
  
  beginShape();
  
  wrapAL.plot();
  
  endShape();
  
  // Add text layer name
  cam.beginHUD();
  
  fill(255);
  textAlign(LEFT);
  textSize(12);
  text(layerName, 40, 200-layerI*20);
  
  cp5.draw();
  cam.endHUD();
  
  popMatrix();
}

void keyReleased() {
  if (key == 'r') {
    drawRace = true;
    drawDOW = false;
    drawMode = false;
  }else if(key == 'd') {
    drawRace = false;
    drawDOW = true;
    drawMode = false;
  } else if(key == 'm') {
    drawRace = false;
    drawDOW = false;
    drawMode = true;
  }
}

void keyPressed() {
  if(key == 'h') {
    addHull = !addHull;
  }
}
